@@ -2,7 +2,7 @@
   +----------------------------------------------------------------------+
   | APC                                                                  |
   +----------------------------------------------------------------------+
-  | Copyright (c) 2006 The PHP Group                                     |
+  | Copyright (c) 2006-2008 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -28,7 +28,7 @@
 
  */
 
-/* $Id: apc_main.c,v 3.102 2007/10/05 23:06:56 gopalv Exp $ */
+/* $Id: apc_main.c,v 3.134 2008/11/26 07:48:39 gopalv Exp $ */
 
 #include "apc_php.h"
 #include "apc_main.h"
@@ -40,15 +40,11 @@
 #include "apc_sma.h"
 #include "apc_stack.h"
 #include "apc_zend.h"
+#include "apc_pool.h"
 #include "SAPI.h"
-#if PHP_API_VERSION <= 20020918
-#if HAVE_APACHE
-#ifdef APC_PHP4_STAT
-#undef XtOffsetOf
-#include "httpd.h"
-#endif
-#endif
-#endif
+#include "php_scandir.h"
+#include "ext/standard/php_var.h"
+#include "ext/standard/md5.h"
 
 /* {{{ module variables */
 
@@ -69,16 +65,22 @@ static zend_compile_t* set_compile_hook(zend_compile_t *ptr)
 /* }}} */
 
 /* {{{ install_function */
-static int install_function(apc_function_t fn TSRMLS_DC)
+static int install_function(apc_function_t fn, apc_context_t* ctxt TSRMLS_DC)
 {
-    int status =
-        zend_hash_add(EG(function_table),
+    zend_function *func;
+    int status;
+
+    func = apc_copy_function_for_execution(fn.function, ctxt);
+
+    status =  zend_hash_add(EG(function_table),
                       fn.name,
                       fn.name_len+1,
-                      apc_copy_function_for_execution(fn.function),
+                      func,
                       sizeof(fn.function[0]),
                       NULL);
 
+    efree(func);
+
     if (status == FAILURE) {
         /* apc_eprint("Cannot redeclare %s()", fn.name); */
     }
@@ -88,14 +90,12 @@ static int install_function(apc_function_t fn TSRMLS_DC)
 /* }}} */
 
 /* {{{ install_class */
-static int install_class(apc_class_t cl TSRMLS_DC)
+static int install_class(apc_class_t cl, apc_context_t* ctxt TSRMLS_DC)
 {
     zend_class_entry* class_entry = cl.class_entry;
     zend_class_entry* parent = NULL;
     int status;
-#ifdef ZEND_ENGINE_2
     zend_class_entry** allocated_ce = NULL;
-#endif
 
 
     /* Special case for mangled names. Mangled names are unique to a file.
@@ -111,47 +111,43 @@ static int install_class(apc_class_t cl TSRMLS_DC)
             return SUCCESS;
         }
     }
-    
-#ifdef ZEND_ENGINE_2    
+
     /*
      * XXX: We need to free this somewhere...
      */
-    allocated_ce = apc_php_malloc(sizeof(zend_class_entry*));    
+    allocated_ce = apc_php_malloc(sizeof(zend_class_entry*));
 
     if(!allocated_ce) {
         return FAILURE;
     }
 
-    *allocated_ce = 
-#endif        
+    *allocated_ce =
     class_entry =
-        apc_copy_class_entry_for_execution(cl.class_entry,
-                                           cl.is_derived);
+        apc_copy_class_entry_for_execution(cl.class_entry, ctxt);
 
 
     /* restore parent class pointer for compile-time inheritance */
     if (cl.parent_name != NULL) {
-#ifdef ZEND_ENGINE_2    
         zend_class_entry** parent_ptr = NULL;
         /*
-         * zend_lookup_class has to be due to presence of __autoload, 
-         * just looking up the EG(class_table) is not enough in php5!
-         * Even more dangerously, thanks to __autoload and people using
-         * class names as filepaths for inclusion, this has to be case
-         * sensitive. zend_lookup_class automatically does a case_fold
-         * internally, but passes the case preserved version to __autoload.
-         * Aside: Do NOT pass *strlen(cl.parent_name)+1* because 
-         * zend_lookup_class does it internally anyway!
+         * __autoload brings in the old issues with mixed inheritance.
+         * When a statically inherited class triggers autoload, it runs
+         * afoul of a potential require_once "parent.php" in the previous 
+         * line, which when executed provides the parent class, but right
+         * now goes and hits __autoload which could fail. 
+         * 
+         * missing parent == re-compile. 
+         *
+         * whether __autoload is enabled or not, because __autoload errors
+         * cause php to die.
+         *
+         * Aside: Do NOT pass *strlen(cl.parent_name)+1* because
+         * zend_lookup_class_ex does it internally anyway!
          */
-        status = zend_lookup_class(cl.parent_name,
-                                    strlen(cl.parent_name),
+        status = zend_lookup_class_ex(cl.parent_name,
+                                    strlen(cl.parent_name), 
+                                    0,
                                     &parent_ptr TSRMLS_CC);
-#else
-        status = zend_hash_find(EG(class_table),
-                                cl.parent_name,
-                                strlen(cl.parent_name)+1,
-                                (void**) &parent);
-#endif
         if (status == FAILURE) {
             if(APCG(report_autofilter)) {
                 apc_wprint("Dynamic inheritance detected for class %s", cl.name);
@@ -160,39 +156,24 @@ static int install_class(apc_class_t cl TSRMLS_DC)
             return status;
         }
         else {
-#ifdef ZEND_ENGINE_2            
             parent = *parent_ptr;
-#endif 
             class_entry->parent = parent;
-#ifdef ZEND_ENGINE_2
             zend_do_inheritance(class_entry, parent TSRMLS_CC);
-#else
-            zend_do_inheritance(class_entry, parent);
-#endif
         }
 
 
     }
 
-#ifdef ZEND_ENGINE_2                           
     status = zend_hash_add(EG(class_table),
                            cl.name,
                            cl.name_len+1,
                            allocated_ce,
                            sizeof(zend_class_entry*),
                            NULL);
-#else                           
-    status = zend_hash_add(EG(class_table),
-                           cl.name,
-                           cl.name_len+1,
-                           class_entry,
-                           sizeof(zend_class_entry),
-                           NULL);
-#endif                           
 
     if (status == FAILURE) {
         apc_eprint("Cannot redeclare class %s", cl.name);
-    } 
+    }
     return status;
 }
 /* }}} */
@@ -202,18 +183,12 @@ static int uninstall_class(apc_class_t cl TSRMLS_DC)
 {
     int status;
 
-#ifdef ZEND_ENGINE_2                           
     status = zend_hash_del(EG(class_table),
                            cl.name,
                            cl.name_len+1);
-#else                           
-    status = zend_hash_del(EG(class_table),
-                           cl.name,
-                           cl.name_len+1);
-#endif                           
     if (status == FAILURE) {
         apc_eprint("Cannot delete class %s", cl.name);
-    } 
+    }
     return status;
 }
 /* }}} */
@@ -231,7 +206,8 @@ static int compare_file_handles(void* a, void* b)
 
 /* {{{ cached_compile */
 static zend_op_array* cached_compile(zend_file_handle* h,
-                                        int type TSRMLS_DC)
+                                        int type,
+                                        apc_context_t* ctxt TSRMLS_DC)
 {
     apc_cache_entry_t* cache_entry;
     int i, ii;
@@ -241,7 +217,7 @@ static zend_op_array* cached_compile(zend_file_handle* h,
 
     if (cache_entry->data.file.classes) {
         for (i = 0; cache_entry->data.file.classes[i].class_entry != NULL; i++) {
-            if(install_class(cache_entry->data.file.classes[i] TSRMLS_CC) == FAILURE) {
+            if(install_class(cache_entry->data.file.classes[i], ctxt TSRMLS_CC) == FAILURE) {
                 goto default_compile;
             }
         }
@@ -249,12 +225,12 @@ static zend_op_array* cached_compile(zend_file_handle* h,
 
     if (cache_entry->data.file.functions) {
         for (i = 0; cache_entry->data.file.functions[i].function != NULL; i++) {
-            install_function(cache_entry->data.file.functions[i] TSRMLS_CC);
+            install_function(cache_entry->data.file.functions[i], ctxt TSRMLS_CC);
         }
     }
 
 
-    return apc_copy_op_array_for_execution(NULL, cache_entry->data.file.op_array TSRMLS_CC);
+    return apc_copy_op_array_for_execution(NULL, cache_entry->data.file.op_array, ctxt TSRMLS_CC);
 
 default_compile:
 
@@ -267,16 +243,22 @@ default_compile:
             uninstall_class(cache_entry->data.file.classes[ii] TSRMLS_CC);
         }
     }
-    
+
     apc_stack_pop(APCG(cache_stack)); /* pop out cache_entry */
-    
+
     apc_cache_release(apc_cache, cache_entry);
 
     /* cannot free up cache data yet, it maybe in use */
-    
-    zend_llist_del_element(&CG(open_files), h, compare_file_handles); /* XXX: kludge */
-    
+
+    zend_llist_del_element(&CG(open_files), h, compare_file_handles); /* We leak fds without this hack */
+
+    /* WARNING: zend_llist shallow copies - so element delete via the 
+     * zend_file_handle_dtor leaves h->opened_path dangling onto bad memory.
+     */
+
+    h->opened_path = NULL;
     h->type = ZEND_HANDLE_FILENAME;
+    if(h->free_filename) h->filename = NULL;
 
     return NULL;
 }
@@ -296,11 +278,11 @@ static zend_op_array* my_compile_file(zend_file_handle* h,
     apc_class_t* alloc_classes;
     time_t t;
     char *path;
-    size_t mem_size;
+    apc_context_t ctxt = {0,};
 
-    if (!APCG(enabled) || (apc_cache_busy(apc_cache) && !APCG(localcache))) {
-		return old_compile_file(h, type TSRMLS_CC);
-	}
+    if (!APCG(enabled) || apc_cache_busy(apc_cache)) {
+        return old_compile_file(h, type TSRMLS_CC);
+    }
 
     /* check our regular expression filters */
     if (APCG(filters) && apc_compiled_filters) {
@@ -312,15 +294,8 @@ static zend_op_array* my_compile_file(zend_file_handle* h,
         return old_compile_file(h, type TSRMLS_CC);
     }
 
-#if PHP_API_VERSION < 20041225
-#if HAVE_APACHE && defined(APC_PHP4_STAT)
-    t = ((request_rec *)SG(server_context))->request_time;
-#else 
-    t = time(0);
-#endif
-#else 
+
     t = sapi_get_request_time(TSRMLS_C);
-#endif
 
 #ifdef __DEBUG_APC__
     fprintf(stderr,"1. h->opened_path=[%s]  h->filename=[%s]\n", h->opened_path?h->opened_path:"null",h->filename);
@@ -333,64 +308,59 @@ static zend_op_array* my_compile_file(zend_file_handle* h,
 
 
     if(!APCG(force_file_update)) {
-        if(APCG(localcache)) {
-            /* search for the file in the local cache */
-            cache_entry = apc_local_cache_find(APCG(lcache), key, t);
-        } else {
-            /* search for the file in the cache */
-            cache_entry = apc_cache_find(apc_cache, key, t);
-        }
+        /* search for the file in the cache */
+        cache_entry = apc_cache_find(apc_cache, key, t);
+        ctxt.force_update = 0;
     } else {
         cache_entry = NULL;
+        ctxt.force_update = 1;
     }
 
     if (cache_entry != NULL) {
         int dummy = 1;
+
+        ctxt.pool = apc_pool_create(APC_UNPOOL, apc_php_malloc, apc_php_free);
+        ctxt.copy = APC_COPY_OUT_OPCODE;
+
         if (h->opened_path == NULL) {
             h->opened_path = estrdup(cache_entry->data.file.filename);
         }
         zend_hash_add(&EG(included_files), h->opened_path, strlen(h->opened_path)+1, (void *)&dummy, sizeof(int), NULL);
-        zend_llist_add_element(&CG(open_files), h); /* XXX kludge */
+
+        zend_llist_add_element(&CG(open_files), h); /* We leak fds without this hack */
+
         apc_stack_push(APCG(cache_stack), cache_entry);
-        op_array = cached_compile(h, type TSRMLS_CC);
+        op_array = cached_compile(h, type, &ctxt TSRMLS_CC);
         if(op_array) {
+#ifdef APC_FILEHITS
+            /* If the file comes from the cache, add it to the global request file list */
+            add_next_index_string(APCG(filehits), h->filename, 1);
+#endif
+            /* this is an unpool, which has no cleanup - this only free's the pool header */
+            apc_pool_destroy(ctxt.pool);
             return op_array;
         }
         if(APCG(report_autofilter)) {
-            apc_wprint("Recompiling %s", h->opened_path);
+            apc_wprint("Recompiling %s", cache_entry->data.file.filename);
         }
         /* TODO: check what happens with EG(included_files) */
     }
-    
-    if(apc_cache_busy(apc_cache) && APCG(localcache)) {
-        /* possibly local cache returned NULL because cache is busy */
-		return old_compile_file(h, type TSRMLS_CC);
-	}
 
     /* remember how many functions and classes existed before compilation */
     num_functions = zend_hash_num_elements(CG(function_table));
     num_classes   = zend_hash_num_elements(CG(class_table));
-    
+
     /* compile the file using the default compile function */
     op_array = old_compile_file(h, type TSRMLS_CC);
     if (op_array == NULL) {
         return NULL;
     }
-    /*
-     * Basically this will cause a file only to be cached on a percentage 
-     * of the attempts.  This is to avoid cache slams when starting up a
-     * very busy server or when modifying files on a very busy live server.
-     * There is no point having many processes all trying to cache the same
-     * file at the same time.  By introducing a chance of being cached
-     * we theoretically cut the cache slam problem by the given percentage.
-     * For example if apc.slam_defense is set to 66 then 2/3 of the attempts
-     * to cache an uncached file will be ignored.
-     */
-    if(APCG(slam_defense)) {
-        if(APCG(slam_rand)==-1) {
-            APCG(slam_rand) = (int)(100.0*rand()/(RAND_MAX+1.0));
-        }
-        if(APCG(slam_rand) < APCG(slam_defense)) {
+
+    /* check our regular expression filters */
+    if (APCG(filters) && apc_compiled_filters && h->opened_path) {
+        int ret = apc_regex_match_array(apc_compiled_filters, h->opened_path);
+        if(ret == APC_NEGATIVE_MATCH || (ret != APC_POSITIVE_MATCH && !APCG(cache_by_default))) {
+            /* never cache, never find */
             return op_array;
         }
     }
@@ -402,8 +372,8 @@ static zend_op_array* my_compile_file(zend_file_handle* h,
         if(!strcmp(SG(request_info).path_translated, h->filename)) {
             tmp_buf = sapi_get_stat(TSRMLS_C);  /* Apache has already done this stat() for us */
         }
-        if(tmp_buf) { 
-            fileinfo.st_buf = *tmp_buf;
+        if(tmp_buf) {
+            fileinfo.st_buf.sb = *tmp_buf;
         } else {
             if (apc_search_paths(h->filename, PG(include_path), &fileinfo) != 0) {
 #ifdef __DEBUG_APC__
@@ -412,7 +382,7 @@ static zend_op_array* my_compile_file(zend_file_handle* h,
                 return op_array;
             }
         }
-        key.mtime = fileinfo.st_buf.st_mtime;
+        key.mtime = fileinfo.st_buf.sb.st_mtime;
     }
 
     HANDLE_BLOCK_INTERRUPTIONS();
@@ -426,47 +396,47 @@ static zend_op_array* my_compile_file(zend_file_handle* h,
     }
 #endif
 
-    mem_size = 0;
-    APCG(mem_size_ptr) = &mem_size;
-    if(!(alloc_op_array = apc_copy_op_array(NULL, op_array, apc_sma_malloc, apc_sma_free TSRMLS_CC))) {
-        apc_cache_expunge(apc_cache,t);
-        apc_cache_expunge(apc_user_cache,t);
-        APCG(mem_size_ptr) = NULL;
-#if NONBLOCKING_LOCK_AVAILABLE
-        if(APCG(write_lock)) {
-            apc_cache_write_unlock(apc_cache);
+    if(APCG(file_md5)) {
+        int n;
+        unsigned char buf[1024];
+        PHP_MD5_CTX context;
+        php_stream *stream;
+        char *filename;
+
+        if(h->opened_path) {
+            filename = h->opened_path;
+        } else {
+            filename = h->filename;
+        }
+        stream = php_stream_open_wrapper(filename, "rb", REPORT_ERRORS | ENFORCE_SAFE_MODE, NULL);
+        if(stream) {
+            PHP_MD5Init(&context);
+            while((n = php_stream_read(stream, (char*)buf, sizeof(buf))) > 0) {
+                PHP_MD5Update(&context, buf, n);
+            }
+            PHP_MD5Final(key.md5, &context);
+            php_stream_close(stream);
+            if(n<0) {
+                apc_wprint("Error while reading '%s' for md5 generation.", filename);
+            }
+        } else {
+            apc_wprint("Unable to open '%s' for md5 generation.", filename);
         }
-#endif
-        HANDLE_UNBLOCK_INTERRUPTIONS();
-        return op_array;
     }
-    
-    if(!(alloc_functions = apc_copy_new_functions(num_functions, apc_sma_malloc, apc_sma_free TSRMLS_CC))) {
-        apc_free_op_array(alloc_op_array, apc_sma_free);
-        apc_cache_expunge(apc_cache,t);
-        apc_cache_expunge(apc_user_cache,t);
-        APCG(mem_size_ptr) = NULL;
-#if NONBLOCKING_LOCK_AVAILABLE
-        if(APCG(write_lock)) {
-            apc_cache_write_unlock(apc_cache);
-        }
-#endif
-        HANDLE_UNBLOCK_INTERRUPTIONS();
-        return op_array;
+
+    APCG(current_cache) = apc_cache;
+    ctxt.pool = apc_pool_create(APC_MEDIUM_POOL, apc_sma_malloc, apc_sma_free);
+    ctxt.copy = APC_COPY_IN_OPCODE;
+
+    if(!(alloc_op_array = apc_copy_op_array(NULL, op_array, &ctxt TSRMLS_CC))) {
+        goto freepool;
     }
-    if(!(alloc_classes = apc_copy_new_classes(op_array, num_classes, apc_sma_malloc, apc_sma_free TSRMLS_CC))) {
-        apc_free_op_array(alloc_op_array, apc_sma_free);
-        apc_free_functions(alloc_functions, apc_sma_free);
-        apc_cache_expunge(apc_cache,t);
-        apc_cache_expunge(apc_user_cache,t);
-        APCG(mem_size_ptr) = NULL;
-#if NONBLOCKING_LOCK_AVAILABLE
-        if(APCG(write_lock)) {
-            apc_cache_write_unlock(apc_cache);
-        }
-#endif
-        HANDLE_UNBLOCK_INTERRUPTIONS();
-        return op_array;
+
+    if(!(alloc_functions = apc_copy_new_functions(num_functions, &ctxt TSRMLS_CC))) {
+        goto freepool;
+    }
+    if(!(alloc_classes = apc_copy_new_classes(op_array, num_classes, &ctxt TSRMLS_CC))) {
+        goto freepool;
     }
 
     path = h->opened_path;
@@ -476,32 +446,18 @@ static zend_op_array* my_compile_file(zend_file_handle* h,
     fprintf(stderr,"2. h->opened_path=[%s]  h->filename=[%s]\n", h->opened_path?h->opened_path:"null",h->filename);
 #endif
 
-    if(!(cache_entry = apc_cache_make_file_entry(path, alloc_op_array, alloc_functions, alloc_classes))) {
-        apc_free_op_array(alloc_op_array, apc_sma_free);
-        apc_free_functions(alloc_functions, apc_sma_free);
-        apc_free_classes(alloc_classes, apc_sma_free);
-        apc_cache_expunge(apc_cache,t);
-        apc_cache_expunge(apc_user_cache,t);
-        APCG(mem_size_ptr) = NULL;
-#if NONBLOCKING_LOCK_AVAILABLE
-        if(APCG(write_lock)) {
-            apc_cache_write_unlock(apc_cache);
-        }
-#endif
-        HANDLE_UNBLOCK_INTERRUPTIONS();
-        return op_array;
+    if(!(cache_entry = apc_cache_make_file_entry(path, alloc_op_array, alloc_functions, alloc_classes, &ctxt))) {
+        goto freepool;
     }
-    APCG(mem_size_ptr) = NULL;
-    cache_entry->mem_size = mem_size;
 
-    if ((ret = apc_cache_insert(apc_cache, key, cache_entry, t)) != 1) {
-        apc_cache_free_entry(cache_entry);
-        if(ret==-1) {
-            apc_cache_expunge(apc_cache,t);
-            apc_cache_expunge(apc_user_cache,t);
-        }
+    if ((ret = apc_cache_insert(apc_cache, key, cache_entry, &ctxt, t)) != 1) {
+freepool:
+        apc_pool_destroy(ctxt.pool);
+        ctxt.pool = NULL;
     }
 
+    APCG(current_cache) = NULL;
+
 #if NONBLOCKING_LOCK_AVAILABLE
     if(APCG(write_lock)) {
         apc_cache_write_unlock(apc_cache);
@@ -513,6 +469,125 @@ static zend_op_array* my_compile_file(zend_file_handle* h,
 }
 /* }}} */
 
+/* {{{ data preload */
+
+extern int _apc_store(char *strkey, int strkey_len, const zval *val, const unsigned int ttl, const int exclusive TSRMLS_DC);
+
+static zval* data_unserialize(const char *filename)
+{
+    zval* retval;
+    long len = 0;
+    struct stat sb;
+    char *contents, *tmp;
+    FILE *fp;
+    php_unserialize_data_t var_hash;
+    TSRMLS_FETCH();
+
+    if(VCWD_STAT(filename, &sb) == -1) {
+        return NULL;
+    }
+
+    fp = fopen(filename, "rb");
+
+    len = sizeof(char)*sb.st_size;
+
+    tmp = contents = malloc(len);
+
+	if(!contents) {
+		return NULL;
+	}
+
+    if(fread(contents, 1, len, fp) < 1) {	
+		free(contents);
+		return NULL;
+	}
+
+    MAKE_STD_ZVAL(retval);
+
+    PHP_VAR_UNSERIALIZE_INIT(var_hash);
+    
+    /* I wish I could use json */
+    if(!php_var_unserialize(&retval, (const unsigned char**)&tmp, (const unsigned char*)(contents+len), &var_hash TSRMLS_CC)) {
+        zval_ptr_dtor(&retval);
+        return NULL;
+    }
+
+    PHP_VAR_UNSERIALIZE_DESTROY(var_hash);
+
+    free(contents);
+    fclose(fp);
+
+    return retval;
+}
+
+static int apc_load_data(const char *data_file TSRMLS_DC)
+{
+    char *p;
+    char key[MAXPATHLEN] = {0,};
+    unsigned int key_len;
+    zval *data;
+
+    p = strrchr(data_file, DEFAULT_SLASH);
+
+    if(p && p[1]) {
+        strlcpy(key, p+1, sizeof(key));
+        p = strrchr(key, '.');
+
+        if(p) {
+            p[0] = '\0';
+            key_len = strlen(key);
+
+            data = data_unserialize(data_file);
+            if(data) {
+                _apc_store(key, key_len, data, 0, 1 TSRMLS_CC);
+            }
+            return 1;
+        }
+    }
+
+    return 0;
+}
+
+static int apc_walk_dir(const char *path TSRMLS_DC)
+{
+    char file[MAXPATHLEN]={0,};
+    int ndir, i;
+    char *p = NULL;
+    struct dirent **namelist = NULL;
+
+    if ((ndir = php_scandir(path, &namelist, 0, php_alphasort)) > 0)
+    {
+        for (i = 0; i < ndir; i++)
+        {
+            /* check for extension */
+            if (!(p = strrchr(namelist[i]->d_name, '.'))
+                    || (p && strcmp(p, ".data")))
+            {
+                free(namelist[i]);
+                continue;
+            }
+            snprintf(file, MAXPATHLEN, "%s%c%s",
+                    path, DEFAULT_SLASH, namelist[i]->d_name);
+            if(!apc_load_data(file TSRMLS_CC))
+            {
+                /* print error */
+            }
+            free(namelist[i]);
+        }
+        free(namelist);
+    }
+
+    return 1;
+}
+
+void apc_data_preload(TSRMLS_D)
+{
+    if(!APCG(preload_path)) return;
+
+    apc_walk_dir(APCG(preload_path) TSRMLS_CC);
+}
+/* }}} */
+
 /* {{{ module init and shutdown */
 
 int apc_module_init(int module_number TSRMLS_DC)
@@ -526,12 +601,17 @@ int apc_module_init(int module_number TSRMLS_DC)
     apc_cache = apc_cache_create(APCG(num_files_hint), APCG(gc_ttl), APCG(ttl));
     apc_user_cache = apc_cache_create(APCG(user_entries_hint), APCG(gc_ttl), APCG(user_ttl));
 
-    apc_compiled_filters = apc_regex_compile_array(APCG(filters));
+    apc_compiled_filters = apc_regex_compile_array(APCG(filters) TSRMLS_CC);
 
     /* override compilation */
     old_compile_file = zend_compile_file;
     zend_compile_file = my_compile_file;
     REGISTER_LONG_CONSTANT("\000apc_magic", (long)&set_compile_hook, CONST_PERSISTENT | CONST_CS);
+    REGISTER_LONG_CONSTANT("\000apc_compile_file", (long)&my_compile_file, CONST_PERSISTENT | CONST_CS);
+
+    apc_pool_init();
+
+    apc_data_preload(TSRMLS_C);
 
     APCG(initialized) = 1;
     return 0;
@@ -545,7 +625,7 @@ int apc_module_shutdown(TSRMLS_D)
     /* restore compilation */
     zend_compile_file = old_compile_file;
 
-    /* 
+    /*
      * In case we got interrupted by a SIGTERM or something else during execution
      * we may have cache entries left on the stack that we need to check to make
      * sure that any functions or classes these may have added to the global function
@@ -588,45 +668,17 @@ int apc_module_shutdown(TSRMLS_D)
 /* {{{ process init and shutdown */
 int apc_process_init(int module_number TSRMLS_DC)
 {
-    int minttl = (APCG(gc_ttl) >  APCG(ttl) ? APCG(ttl) : APCG(gc_ttl))/2;
-    int size = APCG(localcache_size);
-    if(APCG(initialized) && APCG(localcache)) {
-        /* TTL is 2 mins by default */
-        APCG(lcache) = apc_local_cache_create(apc_cache, size, minttl ? minttl : 120); 
-    }
     return 0;
 }
 
 int apc_process_shutdown(TSRMLS_D)
 {
-    if(APCG(initialized) && APCG(localcache) && APCG(lcache)) {
-        apc_local_cache_destroy(APCG(lcache));
-        APCG(lcache) = NULL;
-    }
-	return 0;
-}
-/* }}} */
-
-/* {{{ request init and shutdown */
-
-int apc_request_init(TSRMLS_D)
-{
-    apc_stack_clear(APCG(cache_stack));
-    APCG(slam_rand) = -1;
-    APCG(copied_zvals) = NULL;
     return 0;
 }
-
-int apc_request_shutdown(TSRMLS_D)
-{
-    apc_deactivate(TSRMLS_C);
-    return 0;
-}
-
 /* }}} */
 
 /* {{{ apc_deactivate */
-void apc_deactivate(TSRMLS_D)
+static void apc_deactivate(TSRMLS_D)
 {
     /* The execution stack was unwound, which prevented us from decrementing
      * the reference counts on active cache entries in `my_execute`.
@@ -635,18 +687,14 @@ void apc_deactivate(TSRMLS_D)
         int i;
         zend_class_entry* zce = NULL;
         void ** centry = (void*)(&zce);
-#ifdef ZEND_ENGINE_2
         zend_class_entry** pzce = NULL;
-#endif
-        
+
         apc_cache_entry_t* cache_entry =
             (apc_cache_entry_t*) apc_stack_pop(APCG(cache_stack));
 
         if (cache_entry->data.file.classes) {
             for (i = 0; cache_entry->data.file.classes[i].class_entry != NULL; i++) {
-#ifdef ZEND_ENGINE_2
                 centry = (void**)&pzce; /* a triple indirection to get zend_class_entry*** */
-#endif
                 if(zend_hash_find(EG(class_table), 
                     cache_entry->data.file.classes[i].name,
                     cache_entry->data.file.classes[i].name_len+1,
@@ -658,29 +706,53 @@ void apc_deactivate(TSRMLS_D)
                     continue;
                 }
 
-#ifdef ZEND_ENGINE_2
                 zce = *pzce;
-#endif
                 zend_hash_del(EG(class_table),
                     cache_entry->data.file.classes[i].name,
                     cache_entry->data.file.classes[i].name_len+1);
-                
+
                 apc_free_class_entry_after_execution(zce);
             }
         }
         apc_cache_release(apc_cache, cache_entry);
     }
-    if(APCG(localcache)) {
-        apc_local_cache_cleanup(APCG(lcache)); 
-    }
 }
 /* }}} */
 
+/* {{{ request init and shutdown */
+
+int apc_request_init(TSRMLS_D)
+{
+    apc_stack_clear(APCG(cache_stack));
+    APCG(copied_zvals) = NULL;
+
+#ifdef APC_FILEHITS
+    ALLOC_INIT_ZVAL(APCG(filehits));
+    array_init(APCG(filehits));
+#endif
+
+    return 0;
+}
+
+int apc_request_shutdown(TSRMLS_D)
+{
+    apc_deactivate(TSRMLS_C);
+
+#ifdef APC_FILEHITS
+    zval_ptr_dtor(&APCG(filehits));
+#endif
+
+    return 0;
+}
+
+/* }}} */
+
+
 /*
  * Local variables:
  * tab-width: 4
  * c-basic-offset: 4
  * End:
- * vim600: expandtab sw=4 ts=4 sts=4 fdm=marker
+ * vim>600: expandtab sw=4 ts=4 sts=4 fdm=marker
  * vim<600: expandtab sw=4 ts=4 sts=4
  */
