@@ -2,7 +2,7 @@
   +----------------------------------------------------------------------+
   | APC                                                                  |
   +----------------------------------------------------------------------+
-  | Copyright (c) 2006 The PHP Group                                     |
+  | Copyright (c) 2006-2008 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -29,11 +29,13 @@
 
  */
 
-/* $Id: apc_main.h,v 3.9 2007/02/28 01:15:18 gopalv Exp $ */
+/* $Id: apc_main.h,v 3.15 2008/11/04 05:42:11 rasmus Exp $ */
 
 #ifndef APC_MAIN_H
 #define APC_MAIN_H
 
+#include "apc_pool.h"
+
 /*
  * This module provides the primary interface between PHP and APC.
  */
@@ -45,15 +47,20 @@ extern int apc_process_shutdown(TSRMLS_D);
 extern int apc_request_init(TSRMLS_D);
 extern int apc_request_shutdown(TSRMLS_D);
 
-/*
- * apc_deactivate is called by the PHP interpreter when an "exception" is
- * raised (e.g., a call to the exit function) that unwinds the execution
- * stack.
- */
-extern void apc_deactivate();
+typedef enum _apc_copy_type {
+    APC_NO_COPY = 0,
+    APC_COPY_IN_OPCODE,
+    APC_COPY_OUT_OPCODE,
+    APC_COPY_IN_USER,
+    APC_COPY_OUT_USER
+} apc_copy_type;
 
-
-extern const char* apc_version();
+typedef struct _apc_context_t
+{
+    apc_pool *pool;
+    apc_copy_type copy;
+    unsigned int force_update:1;
+} apc_context_t;
 
 #endif
 
@@ -62,6 +69,6 @@ extern const char* apc_version();
  * tab-width: 4
  * c-basic-offset: 4
  * End:
- * vim600: expandtab sw=4 ts=4 sts=4 fdm=marker
+ * vim>600: expandtab sw=4 ts=4 sts=4 fdm=marker
  * vim<600: expandtab sw=4 ts=4 sts=4
  */
