@@ -2,7 +2,7 @@
   +----------------------------------------------------------------------+
   | APC                                                                  |
   +----------------------------------------------------------------------+
-  | Copyright (c) 2006 The PHP Group                                     |
+  | Copyright (c) 2006-2008 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -25,7 +25,7 @@
 
  */
 
-/* $Id: apc_mmap.c,v 3.6 2007/05/09 22:15:33 shire Exp $ */
+/* $Id: apc_mmap.c,v 3.13 2008/11/26 07:48:39 gopalv Exp $ */
 
 #include "apc.h"
 
@@ -35,7 +35,7 @@
 #include <sys/types.h>
 #include <sys/mman.h>
 
-/* 
+/*
  * Some operating systems (like FreeBSD) have a MAP_NOSYNC flag that
  * tells whatever update daemons might be running to not flush dirty
  * vm pages to disk unless absolutely necessary.  My guess is that
@@ -46,17 +46,26 @@
 #define MAP_NOSYNC 0
 #endif
 
+/* support for systems where MAP_ANONYMOUS is defined but not MAP_ANON, ie: HP-UX bug #14615 */
+#if !defined(MAP_ANON) && defined(MAP_ANONYMOUS)
+# define MAP_ANON MAP_ANONYMOUS
+#endif
+
 void *apc_mmap(char *file_mask, size_t size)
 {
     void* shmaddr;  /* the shared memory address */
 
     /* If no filename was provided, do an anonymous mmap */
     if(!file_mask || (file_mask && !strlen(file_mask))) {
+#if !defined(MAP_ANON)
+        apc_eprint("Anonymous mmap does not apear to be available on this system (MAP_ANON/MAP_ANONYMOUS).  Please see the apc.mmap_file_mask INI option.");
+#else
         shmaddr = (void *)mmap(NULL, size, PROT_READ | PROT_WRITE, MAP_SHARED | MAP_ANON, -1, 0);
+#endif
     } else {
         int fd;
 
-        /* 
+        /*
          * If the filemask contains .shm we try to do a POSIX-compliant shared memory
          * backed mmap which should avoid synchs on some platforms.  At least on
          * FreeBSD this implies MAP_NOSYNC and on Linux it is equivalent of mmap'ing
@@ -67,7 +76,10 @@ void *apc_mmap(char *file_mask, size_t size)
          * path you want here.
          */
         if(strstr(file_mask,".shm")) {
-            mktemp(file_mask);
+            if(!mktemp(file_mask)) {
+                apc_eprint("apc_mmap: mktemp on %s failed:", file_mask);
+                return (void *)-1;
+			}
             fd = shm_open(file_mask, O_CREAT|O_RDWR, S_IRUSR|S_IWUSR);
             if(fd == -1) {
                 apc_eprint("apc_mmap: shm_open on %s failed:", file_mask);
@@ -95,7 +107,7 @@ void *apc_mmap(char *file_mask, size_t size)
             shmaddr = (void *)mmap(NULL, size, PROT_READ | PROT_WRITE, MAP_SHARED, fd, 0);
             close(fd);
         }
-        /* 
+        /*
          * Otherwise we do a normal filesystem mmap
          */
         else {
@@ -114,7 +126,7 @@ void *apc_mmap(char *file_mask, size_t size)
             unlink(file_mask);
         }
     }
-    if((int)shmaddr == -1) {
+    if((long)shmaddr == -1) {
         apc_eprint("apc_mmap: mmap failed:");
     }
     return shmaddr;
@@ -122,7 +134,9 @@ void *apc_mmap(char *file_mask, size_t size)
 
 void apc_unmap(void* shmaddr, size_t size)
 {
-    munmap(shmaddr, size);
+    if (munmap(shmaddr, size) < 0) {
+        apc_wprint("apc_unmap: munmap failed:");
+    }
 }
 
 #endif
@@ -132,6 +146,6 @@ void apc_unmap(void* shmaddr, size_t size)
  * tab-width: 4
  * c-basic-offset: 4
  * End:
- * vim600: expandtab sw=4 ts=4 sts=4 fdm=marker
+ * vim>600: expandtab sw=4 ts=4 sts=4 fdm=marker
  * vim<600: expandtab sw=4 ts=4 sts=4
  */
