@@ -2,7 +2,7 @@
   +----------------------------------------------------------------------+
   | APC                                                                  |
   +----------------------------------------------------------------------+
-  | Copyright (c) 2006 The PHP Group                                     |
+  | Copyright (c) 2006-2008 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -29,7 +29,7 @@
 
  */
 
-/* $Id: apc_php.h,v 3.10 2006/11/16 20:24:48 gopalv Exp $ */
+/* $Id: apc_php.h,v 3.13 2008/11/04 05:42:11 rasmus Exp $ */
 
 #ifndef APC_PHP_H
 #define APC_PHP_H
@@ -47,6 +47,9 @@
 #include "zend_hash.h"
 #include "zend_extensions.h"
 
+#if ZEND_MODULE_API_NO > 20060613
+#define ZEND_ENGINE_2_3
+#endif
 #if ZEND_MODULE_API_NO > 20050922
 #define ZEND_ENGINE_2_2
 #endif
@@ -66,6 +69,6 @@
  * tab-width: 4
  * c-basic-offset: 4
  * End:
- * vim600: expandtab sw=4 ts=4 sts=4 fdm=marker
+ * vim>600: expandtab sw=4 ts=4 sts=4 fdm=marker
  * vim<600: expandtab sw=4 ts=4 sts=4
  */
