@@ -2,7 +2,7 @@
   +----------------------------------------------------------------------+
   | APC                                                                  |
   +----------------------------------------------------------------------+
-  | Copyright (c) 2006 The PHP Group                                     |
+  | Copyright (c) 2006-2008 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -26,7 +26,7 @@
 
  */
 
-/* $Id: apc_shm.c,v 3.11 2007/05/09 22:15:33 shire Exp $ */
+/* $Id: apc_shm.c,v 3.15 2008/11/04 05:42:11 rasmus Exp $ */
 
 #include "apc_shm.h"
 #include "apc.h"
@@ -55,7 +55,7 @@ int apc_shm_create(const char* pathname, int proj, size_t size)
 
     key = IPC_PRIVATE;
 #ifndef PHP_WIN32
-	/* no ftok yet for win32 */
+    /* no ftok yet for win32 */
     if (pathname != NULL) {
         if ((key = ftok(pathname, proj)) < 0) {
             apc_eprint("apc_shm_create: ftok failed:");
@@ -81,7 +81,7 @@ void* apc_shm_attach(int shmid)
 {
     void* shmaddr;  /* the shared memory address */
 
-    if ((int)(shmaddr = shmat(shmid, 0, 0)) == -1) {
+    if ((long)(shmaddr = shmat(shmid, 0, 0)) == -1) {
         apc_eprint("apc_shm_attach: shmat failed:");
     }
 
@@ -105,6 +105,6 @@ void apc_shm_detach(void* shmaddr)
  * tab-width: 4
  * c-basic-offset: 4
  * End:
- * vim600: expandtab sw=4 ts=4 sts=4 fdm=marker
+ * vim>600: expandtab sw=4 ts=4 sts=4 fdm=marker
  * vim<600: expandtab sw=4 ts=4 sts=4
  */
