@@ -2,7 +2,7 @@
   +----------------------------------------------------------------------+
   | APC                                                                  |
   +----------------------------------------------------------------------+
-  | Copyright (c) 2006 The PHP Group                                     |
+  | Copyright (c) 2006-2008 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -26,10 +26,11 @@
 
  */
 
-/* $Id: php_apc.c,v 3.151 2007/09/18 21:35:46 shire Exp $ */
+/* $Id: php_apc.c,v 3.196 2008/11/26 10:20:47 gopalv Exp $ */
 
 #include "apc_zend.h"
 #include "apc_cache.h"
+#include "apc_iterator.h"
 #include "apc_main.h"
 #include "apc_sma.h"
 #include "apc_lock.h"
@@ -39,13 +40,10 @@
 #include "SAPI.h"
 #include "rfc1867.h"
 #include "php_apc.h"
-#if PHP_API_VERSION <= 20020918
-#if HAVE_APACHE
-#ifdef APC_PHP4_STAT
-#undef XtOffsetOf
-#include "httpd.h"
-#endif
-#endif
+#include "ext/standard/md5.h"
+
+#if HAVE_SIGACTION
+#include "apc_signal.h"
 #endif
 
 /* {{{ PHP_FUNCTION declarations */
@@ -55,17 +53,21 @@ PHP_FUNCTION(apc_sma_info);
 PHP_FUNCTION(apc_store);
 PHP_FUNCTION(apc_fetch);
 PHP_FUNCTION(apc_delete);
+PHP_FUNCTION(apc_delete_file);
 PHP_FUNCTION(apc_compile_file);
 PHP_FUNCTION(apc_define_constants);
 PHP_FUNCTION(apc_load_constants);
 PHP_FUNCTION(apc_add);
+PHP_FUNCTION(apc_inc);
+PHP_FUNCTION(apc_dec);
+PHP_FUNCTION(apc_cas);
 /* }}} */
 
 /* {{{ ZEND_DECLARE_MODULE_GLOBALS(apc) */
 ZEND_DECLARE_MODULE_GLOBALS(apc)
 
 /* True globals */
-apc_cache_t* apc_cache = NULL;       
+apc_cache_t* apc_cache = NULL;
 apc_cache_t* apc_user_cache = NULL;
 void* apc_compiled_filters = NULL;
 
@@ -75,24 +77,21 @@ static void php_apc_init_globals(zend_apc_globals* apc_globals TSRMLS_DC)
     apc_globals->initialized = 0;
     apc_globals->cache_stack = apc_stack_create(0);
     apc_globals->cache_by_default = 1;
-    apc_globals->slam_defense = 0;
-    apc_globals->mem_size_ptr = NULL;
     apc_globals->fpstat = 1;
+    apc_globals->canonicalize = 1;
     apc_globals->stat_ctime = 0;
     apc_globals->write_lock = 1;
     apc_globals->report_autofilter = 0;
     apc_globals->apc_optimize_function = NULL;
 #ifdef MULTIPART_EVENT_FORMDATA
     apc_globals->rfc1867 = 0;
+    memset(&(apc_globals->rfc1867_data), 0, sizeof(apc_rfc1867_data));
 #endif
     apc_globals->copied_zvals = NULL;
-#ifdef ZEND_ENGINE_2
     apc_globals->reserved_offset = -1;
-#endif
-    apc_globals->localcache = 0;
-    apc_globals->localcache_size = 0;
-    apc_globals->lcache = NULL;
     apc_globals->force_file_update = 0;
+    apc_globals->coredump_unmap = 0;
+    apc_globals->preload_path = NULL;
 }
 
 static void php_apc_shutdown_globals(zend_apc_globals* apc_globals TSRMLS_DC)
@@ -107,7 +106,7 @@ static void php_apc_shutdown_globals(zend_apc_globals* apc_globals TSRMLS_DC)
     }
 
     /* the stack should be empty */
-    assert(apc_stack_size(apc_globals->cache_stack) == 0); 
+    assert(apc_stack_size(apc_globals->cache_stack) == 0);
 
     /* apc cleanup */
     apc_stack_destroy(apc_globals->cache_stack);
@@ -132,7 +131,7 @@ static PHP_INI_MH(OnUpdateShmSegments) /* {{{ */
     if(atoi(new_value)!=1) {
         php_error_docref(NULL TSRMLS_CC, E_WARNING, "apc.shm_segments setting ignored in MMAP mode");
     }
-    APCG(shm_segments) = 1; 
+    APCG(shm_segments) = 1;
 #else
     APCG(shm_segments) = atoi(new_value);
 #endif
@@ -163,9 +162,7 @@ static PHP_INI_MH(OnUpdateRfc1867Freq) /* {{{ */
 /* }}} */
 #endif
 
-#ifdef ZEND_ENGINE_2
 #define OnUpdateInt OnUpdateLong
-#endif
 
 PHP_INI_BEGIN()
 STD_PHP_INI_BOOLEAN("apc.enabled",      "1",    PHP_INI_SYSTEM, OnUpdateBool,              enabled,         zend_apc_globals, apc_globals)
@@ -182,11 +179,11 @@ STD_PHP_INI_ENTRY("apc.mmap_file_mask",  NULL,  PHP_INI_SYSTEM, OnUpdateString,
 #endif
 PHP_INI_ENTRY("apc.filters",        NULL,     PHP_INI_SYSTEM, OnUpdate_filters)
 STD_PHP_INI_BOOLEAN("apc.cache_by_default", "1",  PHP_INI_ALL, OnUpdateBool,         cache_by_default, zend_apc_globals, apc_globals)
-STD_PHP_INI_ENTRY("apc.slam_defense", "0",      PHP_INI_SYSTEM, OnUpdateInt,            slam_defense,     zend_apc_globals, apc_globals)
 STD_PHP_INI_ENTRY("apc.file_update_protection", "2", PHP_INI_SYSTEM, OnUpdateInt,file_update_protection,  zend_apc_globals, apc_globals)
 STD_PHP_INI_BOOLEAN("apc.enable_cli", "0",      PHP_INI_SYSTEM, OnUpdateBool,           enable_cli,       zend_apc_globals, apc_globals)
 STD_PHP_INI_ENTRY("apc.max_file_size", "1M",    PHP_INI_SYSTEM, OnUpdateInt,            max_file_size,    zend_apc_globals, apc_globals)
 STD_PHP_INI_BOOLEAN("apc.stat", "1",            PHP_INI_SYSTEM, OnUpdateBool,           fpstat,           zend_apc_globals, apc_globals)
+STD_PHP_INI_BOOLEAN("apc.canonicalize", "1",    PHP_INI_SYSTEM, OnUpdateBool,           canonicalize,     zend_apc_globals, apc_globals)
 STD_PHP_INI_BOOLEAN("apc.stat_ctime", "0",      PHP_INI_SYSTEM, OnUpdateBool,           stat_ctime,       zend_apc_globals, apc_globals)
 STD_PHP_INI_BOOLEAN("apc.write_lock", "1",      PHP_INI_SYSTEM, OnUpdateBool,           write_lock,       zend_apc_globals, apc_globals)
 STD_PHP_INI_BOOLEAN("apc.report_autofilter", "0", PHP_INI_SYSTEM, OnUpdateBool,         report_autofilter,zend_apc_globals, apc_globals)
@@ -195,9 +192,11 @@ STD_PHP_INI_BOOLEAN("apc.rfc1867", "0", PHP_INI_SYSTEM, OnUpdateBool, rfc1867, z
 STD_PHP_INI_ENTRY("apc.rfc1867_prefix", "upload_", PHP_INI_SYSTEM, OnUpdateStringUnempty, rfc1867_prefix, zend_apc_globals, apc_globals)
 STD_PHP_INI_ENTRY("apc.rfc1867_name", "APC_UPLOAD_PROGRESS", PHP_INI_SYSTEM, OnUpdateStringUnempty, rfc1867_name, zend_apc_globals, apc_globals)
 STD_PHP_INI_ENTRY("apc.rfc1867_freq", "0", PHP_INI_SYSTEM, OnUpdateRfc1867Freq, rfc1867_freq, zend_apc_globals, apc_globals)
+STD_PHP_INI_ENTRY("apc.rfc1867_ttl", "3600", PHP_INI_SYSTEM, OnUpdateInt, rfc1867_ttl, zend_apc_globals, apc_globals)
 #endif
-STD_PHP_INI_BOOLEAN("apc.localcache", "0", PHP_INI_SYSTEM, OnUpdateBool, localcache, zend_apc_globals, apc_globals)
-STD_PHP_INI_ENTRY("apc.localcache.size", "512", PHP_INI_SYSTEM, OnUpdateInt, localcache_size,  zend_apc_globals, apc_globals)
+STD_PHP_INI_BOOLEAN("apc.coredump_unmap", "0", PHP_INI_SYSTEM, OnUpdateBool, coredump_unmap, zend_apc_globals, apc_globals)
+STD_PHP_INI_ENTRY("apc.preload_path", (char*)NULL,              PHP_INI_SYSTEM, OnUpdateString,       preload_path,  zend_apc_globals, apc_globals)
+STD_PHP_INI_BOOLEAN("apc.file_md5", "0", PHP_INI_SYSTEM, OnUpdateBool, file_md5,  zend_apc_globals, apc_globals)
 PHP_INI_END()
 
 /* }}} */
@@ -206,26 +205,16 @@ PHP_INI_END()
 static PHP_MINFO_FUNCTION(apc)
 {
     php_info_print_table_start();
-    php_info_print_table_row(2, "APC Support", APCG(enabled) ? "enabled" : "disabled");
-    php_info_print_table_row(2, "Version", APC_VERSION);
+    php_info_print_table_header(2, "APC Support", APCG(enabled) ? "enabled" : "disabled");
+    php_info_print_table_row(2, "Version", PHP_APC_VERSION);
 #if APC_MMAP
     php_info_print_table_row(2, "MMAP Support", "Enabled");
     php_info_print_table_row(2, "MMAP File Mask", APCG(mmap_file_mask));
 #else
     php_info_print_table_row(2, "MMAP Support", "Disabled");
 #endif
-#if APC_SEM_LOCKS
-    php_info_print_table_row(2, "Locking type", "IPC Semaphore");
-#elif APC_FUTEX_LOCKS
-    php_info_print_table_row(2, "Locking type", "Linux Futex Locks");
-#elif APC_PTHREADMUTEX_LOCKS
-    php_info_print_table_row(2, "Locking type", "pthread mutex Locks");
-#elif APC_SPIN_LOCKS
-    php_info_print_table_row(2, "Locking type", "spin Locks");
-#else
-    php_info_print_table_row(2, "Locking type", "File Locks");
-#endif
-    php_info_print_table_row(2, "Revision", "$Revision: 3.151 $");
+    php_info_print_table_row(2, "Locking type", APC_LOCK_TYPE);
+    php_info_print_table_row(2, "Revision", "$Revision: 3.196 $");
     php_info_print_table_row(2, "Build Date", __DATE__ " " __TIME__);
     php_info_print_table_end();
     DISPLAY_INI_ENTRIES();
@@ -261,6 +250,7 @@ static PHP_MINIT_FUNCTION(apc)
                 php_rfc1867_callback = apc_rfc1867_progress;
             }
 #endif
+            apc_iterator_init(module_number TSRMLS_CC);
         }
     }
 
@@ -277,6 +267,9 @@ static PHP_MSHUTDOWN_FUNCTION(apc)
         apc_module_shutdown(TSRMLS_C);
 #ifndef ZTS
         php_apc_shutdown_globals(&apc_globals);
+#endif
+#if HAVE_SIGACTION
+        apc_shutdown_signals();
 #endif
     }
 #ifdef ZTS
@@ -292,6 +285,10 @@ static PHP_RINIT_FUNCTION(apc)
 {
     if(APCG(enabled)) {
         apc_request_init(TSRMLS_C);
+
+#if HAVE_SIGACTION
+        apc_set_signals(TSRMLS_C);
+#endif
     }
     return SUCCESS;
 }
@@ -307,7 +304,7 @@ static PHP_RSHUTDOWN_FUNCTION(apc)
 }
 /* }}} */
 
-/* {{{ proto array apc_cache_info([string type] [, bool limited]) */
+/* {{{ proto array apc_cache_info([string type [, bool limited]]) */
 PHP_FUNCTION(apc_cache_info)
 {
     apc_cache_info_t* info;
@@ -316,6 +313,7 @@ PHP_FUNCTION(apc_cache_info)
     char *cache_type;
     int ct_len;
     zend_bool limited=0;
+    char md5str[33];
 
     if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|sb", &cache_type, &ct_len, &limited) == FAILURE) {
         return;
@@ -324,6 +322,13 @@ PHP_FUNCTION(apc_cache_info)
     if(ZEND_NUM_ARGS()) {
         if(!strcasecmp(cache_type,"user")) {
             info = apc_cache_info(apc_user_cache, limited);
+        } else if(!strcasecmp(cache_type,"filehits")) {
+#ifdef APC_FILEHITS
+            RETVAL_ZVAL(APCG(filehits), 1, 0);
+            return;
+#else
+            RETURN_FALSE;
+#endif
         } else {
             info = apc_cache_info(apc_cache, limited);
         }
@@ -356,8 +361,6 @@ PHP_FUNCTION(apc_cache_info)
 #endif
 #if APC_SEM_LOCKS
     add_assoc_stringl(return_value, "locking_type", "IPC semaphore", sizeof("IPC semaphore")-1, 1);
-#elif APC_FUTEX_LOCKS
-    add_assoc_stringl(return_value, "locking_type", "Linux Futex", sizeof("Linux Futex")-1, 1);
 #elif APC_PTHREADMUTEX_LOCKS
     add_assoc_stringl(return_value, "locking_type", "pthread mutex", sizeof("pthread mutex")-1, 1);
 #elif APC_SPIN_LOCKS
@@ -369,7 +372,7 @@ PHP_FUNCTION(apc_cache_info)
         apc_cache_free_info(info);
         return;
     }
-    
+
     ALLOC_INIT_ZVAL(list);
     array_init(list);
 
@@ -384,6 +387,10 @@ PHP_FUNCTION(apc_cache_info)
             add_assoc_long(link, "device", p->data.file.device);
             add_assoc_long(link, "inode", p->data.file.inode);
             add_assoc_string(link, "type", "file", 1);
+            if(APCG(file_md5)) {
+                make_digest_ex(md5str, p->data.file.md5, 16);
+                add_assoc_string(link, "md5", md5str, 1);
+            }
         } else if(p->type == APC_CACHE_ENTRY_USER) {
             add_assoc_string(link, "info", p->data.user.info, 1);
             add_assoc_long(link, "ttl", (long)p->data.user.ttl);
@@ -414,6 +421,10 @@ PHP_FUNCTION(apc_cache_info)
             add_assoc_long(link, "device", p->data.file.device);
             add_assoc_long(link, "inode", p->data.file.inode);
             add_assoc_string(link, "type", "file", 1);
+            if(APCG(file_md5)) {
+                make_digest_ex(md5str, p->data.file.md5, 16);
+                add_assoc_string(link, "md5", md5str, 1);
+            }
         } else if(p->type == APC_CACHE_ENTRY_USER) {
             add_assoc_string(link, "info", p->data.user.info, 1);
             add_assoc_long(link, "ttl", (long)p->data.user.ttl);
@@ -434,7 +445,7 @@ PHP_FUNCTION(apc_cache_info)
 }
 /* }}} */
 
-/* {{{ proto void apc_clear_cache() */
+/* {{{ proto void apc_clear_cache([string cache]) */
 PHP_FUNCTION(apc_clear_cache)
 {
     char *cache_type;
@@ -522,55 +533,59 @@ PHP_FUNCTION(apc_sma_info)
 }
 /* }}} */
 
+/* {{{ */
+int _apc_update(char *strkey, int strkey_len, apc_cache_updater_t updater, void* data TSRMLS_DC) 
+{
+    HANDLE_BLOCK_INTERRUPTIONS();
+    APCG(current_cache) = apc_user_cache;
+    
+    if (!_apc_cache_user_update(apc_user_cache, strkey, strkey_len, updater, data TSRMLS_CC)) {
+        HANDLE_UNBLOCK_INTERRUPTIONS();
+        return 0;
+    }
+
+    APCG(current_cache) = NULL;
+    HANDLE_UNBLOCK_INTERRUPTIONS();
+
+    return 1;
+}
+/* }}} */
+    
 /* {{{ _apc_store */
 int _apc_store(char *strkey, int strkey_len, const zval *val, const unsigned int ttl, const int exclusive TSRMLS_DC) {
     apc_cache_entry_t *entry;
     apc_cache_key_t key;
     time_t t;
-    size_t mem_size = 0;
+    apc_context_t ctxt={0,};
 
-#if PHP_API_VERSION < 20041225
-#if HAVE_APACHE && defined(APC_PHP4_STAT)
-    t = ((request_rec *)SG(server_context))->request_time;
-#else
-    t = time(0);
-#endif
-#else
     t = sapi_get_request_time(TSRMLS_C);
-#endif
 
     if(!APCG(enabled)) return 0;
 
     HANDLE_BLOCK_INTERRUPTIONS();
 
-    APCG(mem_size_ptr) = &mem_size;
-    if (!(entry = apc_cache_make_user_entry(strkey, strkey_len + 1, val, ttl))) {
-        APCG(mem_size_ptr) = NULL;
-        apc_cache_expunge(apc_cache,t);
-        apc_cache_expunge(apc_user_cache,t);
+    APCG(current_cache) = apc_user_cache;
+
+    ctxt.pool = apc_pool_create(APC_SMALL_POOL, apc_sma_malloc, apc_sma_free);
+    ctxt.copy = APC_COPY_IN_USER;
+    ctxt.force_update = 0;
+
+    if (!(entry = apc_cache_make_user_entry(strkey, strkey_len, val, &ctxt, ttl))) {
+        goto freepool;
+    }
+
+    if (!apc_cache_make_user_key(&key, strkey, strkey_len, t)) {
+        goto freepool;
+    }
+
+    if (!apc_cache_user_insert(apc_user_cache, key, entry, &ctxt, t, exclusive TSRMLS_CC)) {
+freepool:
+        apc_pool_destroy(ctxt.pool);
         HANDLE_UNBLOCK_INTERRUPTIONS();
         return 0;
     }
 
-    if (!apc_cache_make_user_key(&key, strkey, strkey_len + 1, t)) {
-        APCG(mem_size_ptr) = NULL;
-        apc_cache_free_entry(entry);
-        apc_cache_expunge(apc_cache,t);
-        apc_cache_expunge(apc_user_cache,t);
-        HANDLE_UNBLOCK_INTERRUPTIONS();
-        return 0;
-    }
-
-    if (!apc_cache_user_insert(apc_user_cache, key, entry, t, exclusive TSRMLS_CC)) {
-        APCG(mem_size_ptr) = NULL;
-        apc_cache_free_entry(entry);
-        apc_cache_expunge(apc_cache,t);
-        apc_cache_expunge(apc_user_cache,t);
-        HANDLE_UNBLOCK_INTERRUPTIONS();
-        return 0;
-    }
-
-    APCG(mem_size_ptr) = NULL;
+    APCG(current_cache) = NULL;
 
     HANDLE_UNBLOCK_INTERRUPTIONS();
 
@@ -578,7 +593,7 @@ int _apc_store(char *strkey, int strkey_len, const zval *val, const unsigned int
 }
 /* }}} */
 
-/* {{{ proto int apc_store(string key, zval var [, ttl ])
+/* {{{ proto int apc_store(string key, mixed var [, long ttl ])
  */
 PHP_FUNCTION(apc_store) {
     zval *val;
@@ -597,7 +612,7 @@ PHP_FUNCTION(apc_store) {
 }
 /* }}} */
 
-/* {{{ proto int apc_add(string key, zval var [, ttl ])
+/* {{{ proto int apc_add(string key, mixed var [, long ttl ])
  */
 PHP_FUNCTION(apc_add) {
     zval *val;
@@ -616,38 +631,120 @@ PHP_FUNCTION(apc_add) {
 }
 /* }}} */
 
+/* {{{ inc_updater */
+
+struct _inc_update_args {
+    long step;
+    long lval;
+};
+
+static int inc_updater(apc_cache_t* cache, apc_cache_entry_t* entry, void* data) {
+
+    struct _inc_update_args *args = (struct _inc_update_args*) data;
+    
+    zval* val = entry->data.user.val;
+
+    if(Z_TYPE_P(val) == IS_LONG) {
+        val->value.lval += args->step;
+        args->lval = val->value.lval;
+        return 1;
+    }
+
+    return 0;
+}
+/* }}} */
+
+/* {{{ proto long apc_inc(string key [, long step [, bool& success]])
+ */
+PHP_FUNCTION(apc_inc) {
+    char *strkey;
+    int strkey_len;
+    struct _inc_update_args args = {1L, -1};
+    zval *success = NULL;
+
+    if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s|lz", &strkey, &strkey_len, &(args.step), &success) == FAILURE) {
+        return;
+    }
+
+    if(_apc_update(strkey, strkey_len, inc_updater, &args TSRMLS_CC)) {
+        if(success) ZVAL_TRUE(success);
+        RETURN_LONG(args.lval);
+    }
+    
+    if(success) ZVAL_FALSE(success);
+    
+    RETURN_FALSE;
+}
+/* }}} */
+
+/* {{{ proto long apc_dec(string key [, long step [, long &success]])
+ */
+PHP_FUNCTION(apc_dec) {
+    char *strkey;
+    int strkey_len;
+    struct _inc_update_args args = {1L, -1};
+    zval *success = NULL;
+
+    if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s|lz", &strkey, &strkey_len, &(args.step), &success) == FAILURE) {
+        return;
+    }
+
+    args.step = args.step * -1;
+
+    if(_apc_update(strkey, strkey_len, inc_updater, &args TSRMLS_CC)) {
+        if(success) ZVAL_TRUE(success);
+        RETURN_LONG(args.lval);
+    }
+    
+    if(success) ZVAL_FALSE(success);
+    
+    RETURN_FALSE;
+}
+/* }}} */
+
+/* {{{ cas_updater */
+static int cas_updater(apc_cache_t* cache, apc_cache_entry_t* entry, void* data) {
+    long* vals = ((long*)data);
+    long old = vals[0];
+    long new = vals[1];
+    zval* val = entry->data.user.val;
+
+    if(Z_TYPE_P(val) == IS_LONG) {
+        if(val->value.lval == old) {
+            val->value.lval = new;
+            return 1;
+        }
+    }
+
+    return 0;
+}
+/* }}} */
+
+/* {{{ proto int apc_cas(string key, long old, long new)
+ */
+PHP_FUNCTION(apc_cas) {
+    char *strkey;
+    int strkey_len;
+    long vals[2];
+
+    if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "sll", &strkey, &strkey_len, &vals[0], &vals[1]) == FAILURE) {
+        return;
+    }
+
+    if(_apc_update(strkey, strkey_len, cas_updater, &vals TSRMLS_CC)) RETURN_TRUE;
+    RETURN_FALSE;
+}
+/* }}} */
+
 void *apc_erealloc_wrapper(void *ptr, size_t size) {
     return _erealloc(ptr, size, 0 ZEND_FILE_LINE_CC ZEND_FILE_LINE_EMPTY_CC);
 }
 
-/* {{{ RETURN_ZVAL for php4 */
-#if !defined(ZEND_ENGINE_2) && !defined(RETURN_ZVAL)
-#define RETURN_ZVAL(zv, copy, dtor) { RETVAL_ZVAL(zv, copy, dtor); return; } 
-#define RETVAL_ZVAL(zv, copy, dtor)     ZVAL_ZVAL(return_value, zv, copy, dtor)
-#define ZVAL_ZVAL(z, zv, copy, dtor) {  \
-        int is_ref, refcount;           \
-        is_ref = (z)->is_ref;           \
-        refcount = (z)->refcount;       \
-        *(z) = *(zv);                   \
-        if (copy) {                     \
-            zval_copy_ctor(z);          \
-        }                               \
-        if (dtor) {                     \
-            if (!copy) {                \
-                ZVAL_NULL(zv);          \
-            }                           \
-            zval_ptr_dtor(&zv);         \
-        }                               \
-        (z)->is_ref = is_ref;           \
-        (z)->refcount = refcount;       \
-    }
-#endif
-/* }}} */
-
-/* {{{ proto mixed apc_fetch(mixed key)
+/* {{{ proto mixed apc_fetch(mixed key[, bool &success])
  */
 PHP_FUNCTION(apc_fetch) {
     zval *key;
+    zval *success = NULL;
     HashTable *hash;
     HashPosition hpos;
     zval **hentry;
@@ -657,27 +754,28 @@ PHP_FUNCTION(apc_fetch) {
     int strkey_len;
     apc_cache_entry_t* entry;
     time_t t;
+    apc_context_t ctxt = {0,};
 
     if(!APCG(enabled)) RETURN_FALSE;
 
-    if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "z", &key) == FAILURE) {
+    if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "z|z", &key, &success) == FAILURE) {
         return;
     }
 
-#if PHP_API_VERSION < 20041225
-#if HAVE_APACHE && defined(APC_PHP4_STAT)
-    t = ((request_rec *)SG(server_context))->request_time;
-#else 
-    t = time(0);
-#endif
-#else
+    ctxt.pool = apc_pool_create(APC_UNPOOL, apc_php_malloc, apc_php_free);
+    ctxt.copy = APC_COPY_OUT_USER;
+    ctxt.force_update = 0;
+
     t = sapi_get_request_time(TSRMLS_C);
-#endif
+
+    if (success) {
+        ZVAL_BOOL(success, 0);
+    }
 
     if(Z_TYPE_P(key) != IS_STRING && Z_TYPE_P(key) != IS_ARRAY) {
         convert_to_string(key);
     }
-    
+
     if(Z_TYPE_P(key) == IS_STRING) {
         strkey = Z_STRVAL_P(key);
         strkey_len = Z_STRLEN_P(key);
@@ -685,10 +783,10 @@ PHP_FUNCTION(apc_fetch) {
         entry = apc_cache_user_find(apc_user_cache, strkey, strkey_len + 1, t);
         if(entry) {
             /* deep-copy returned shm zval to emalloc'ed return_value */
-            apc_cache_fetch_zval(return_value, entry->data.user.val, apc_php_malloc, apc_php_free);
+            apc_cache_fetch_zval(return_value, entry->data.user.val, &ctxt);
             apc_cache_release(apc_user_cache, entry);
         } else {
-            RETURN_FALSE;
+            goto freepool;
         }
     } else if(Z_TYPE_P(key) == IS_ARRAY) {
         hash = Z_ARRVAL_P(key);
@@ -698,46 +796,131 @@ PHP_FUNCTION(apc_fetch) {
         while(zend_hash_get_current_data_ex(hash, (void**)&hentry, &hpos) == SUCCESS) {
             if(Z_TYPE_PP(hentry) != IS_STRING) {
                 apc_wprint("apc_fetch() expects a string or array of strings.");
-                RETURN_FALSE;
+                goto freepool;
             }
             entry = apc_cache_user_find(apc_user_cache, Z_STRVAL_PP(hentry), Z_STRLEN_PP(hentry) + 1, t);
             if(entry) {
                 /* deep-copy returned shm zval to emalloc'ed return_value */
                 MAKE_STD_ZVAL(result_entry);
-                apc_cache_fetch_zval(result_entry, entry->data.user.val, apc_php_malloc, apc_php_free);
+                apc_cache_fetch_zval(result_entry, entry->data.user.val, &ctxt);
                 apc_cache_release(apc_user_cache, entry);
                 zend_hash_add(Z_ARRVAL_P(result), Z_STRVAL_PP(hentry), Z_STRLEN_PP(hentry) +1, &result_entry, sizeof(zval*), NULL);
             } /* don't set values we didn't find */
             zend_hash_move_forward_ex(hash, &hpos);
         }
-        RETURN_ZVAL(result, 0, 1);
+        RETVAL_ZVAL(result, 0, 1);
     } else {
         apc_wprint("apc_fetch() expects a string or array of strings.");
+freepool:
+        apc_pool_destroy(ctxt.pool);
         RETURN_FALSE;
     }
 
+    if (success) {
+        ZVAL_BOOL(success, 1);
+    }
+
+    apc_pool_destroy(ctxt.pool);
     return;
 }
 /* }}} */
 
-/* {{{ proto mixed apc_delete(string key)
+
+/* {{{ proto mixed apc_delete(mixed keys)
  */
 PHP_FUNCTION(apc_delete) {
-    char *strkey;
-    int strkey_len;
+    zval *keys;
 
     if(!APCG(enabled)) RETURN_FALSE;
 
-    if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &strkey, &strkey_len) == FAILURE) {
+    if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "z", &keys) == FAILURE) {
         return;
     }
 
-    if(!strkey_len) RETURN_FALSE;
-
-    if(apc_cache_user_delete(apc_user_cache, strkey, strkey_len + 1)) {
-        RETURN_TRUE;
+    if (Z_TYPE_P(keys) == IS_STRING) {
+        if (!Z_STRLEN_P(keys)) RETURN_FALSE;
+        if(apc_cache_user_delete(apc_user_cache, Z_STRVAL_P(keys), Z_STRLEN_P(keys) + 1)) {
+            RETURN_TRUE;
+        } else {
+            RETURN_FALSE;
+        }
+    } else if (Z_TYPE_P(keys) == IS_ARRAY) {
+        HashTable *hash = Z_ARRVAL_P(keys);
+        HashPosition hpos;
+        zval **hentry;
+        array_init(return_value);
+        zend_hash_internal_pointer_reset_ex(hash, &hpos);
+        while(zend_hash_get_current_data_ex(hash, (void**)&hentry, &hpos) == SUCCESS) {
+            if(Z_TYPE_PP(hentry) != IS_STRING) {
+                apc_wprint("apc_delete() expects a string, array of strings, or APCIterator instance.");
+                add_next_index_zval(return_value, *hentry);
+                Z_ADDREF_PP(hentry);
+            } else if(apc_cache_user_delete(apc_user_cache, Z_STRVAL_PP(hentry), Z_STRLEN_PP(hentry) + 1) != 1) {
+                add_next_index_zval(return_value, *hentry);
+                Z_ADDREF_PP(hentry);
+            }
+            zend_hash_move_forward_ex(hash, &hpos);
+        }
+        return;
+    } else if (Z_TYPE_P(keys) == IS_OBJECT) {
+        if (apc_iterator_delete(keys TSRMLS_CC)) {
+            RETURN_TRUE;
+        } else {
+            RETURN_FALSE;
+        }
     } else {
-        RETURN_FALSE;
+        apc_wprint("apc_delete() expects a string, array of strings, or APCIterator instance.");
+    }
+}
+/* }}} */
+
+/* {{{ proto mixed apc_delete_file(mixed keys)
+ *       Deletes the given files from the opcode cache.  
+ *       Accepts a string, array of strings, or APCIterator object. 
+ *       Returns True/False, or for an Array an Array of failed files.
+ */
+PHP_FUNCTION(apc_delete_file) {
+    zval *keys;
+
+    if(!APCG(enabled)) RETURN_FALSE;
+
+    if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "z", &keys) == FAILURE) {
+        return;
+    }
+
+    if (Z_TYPE_P(keys) == IS_STRING) {
+        if (!Z_STRLEN_P(keys)) RETURN_FALSE;
+        if(apc_cache_delete(apc_cache, Z_STRVAL_P(keys), Z_STRLEN_P(keys) + 1) != 1) {
+            RETURN_FALSE;
+        } else {
+            RETURN_TRUE;
+        }
+    } else if (Z_TYPE_P(keys) == IS_ARRAY) {
+        HashTable *hash = Z_ARRVAL_P(keys);
+        HashPosition hpos;
+        zval **hentry;
+        array_init(return_value);
+        zend_hash_internal_pointer_reset_ex(hash, &hpos);
+        while(zend_hash_get_current_data_ex(hash, (void**)&hentry, &hpos) == SUCCESS) {
+            if(Z_TYPE_PP(hentry) != IS_STRING) {
+                apc_wprint("apc_delete_file() expects a string, array of strings, or APCIterator instance.");
+                add_next_index_zval(return_value, *hentry);
+                Z_ADDREF_PP(hentry);
+            } else if(apc_cache_delete(apc_cache, Z_STRVAL_PP(hentry), Z_STRLEN_PP(hentry) + 1) != 1) {
+                add_next_index_zval(return_value, *hentry);
+                Z_ADDREF_PP(hentry);
+            }
+            zend_hash_move_forward_ex(hash, &hpos);
+        }
+        return;
+    } else if (Z_TYPE_P(keys) == IS_OBJECT) {
+        if (apc_iterator_delete(keys TSRMLS_CC)) {
+            RETURN_TRUE;
+        } else {
+            RETURN_FALSE;
+        }
+    } else {
+        apc_wprint("apc_delete_file() expects a string, array of strings, or APCIterator instance.");
     }
 }
 /* }}} */
@@ -776,9 +959,7 @@ static void _apc_define_constants(zval *constants, zend_bool case_sensitive TSRM
         c.flags = case_sensitive;
         c.name = zend_strndup(const_key, const_key_len);
         c.name_len = const_key_len;
-#ifdef ZEND_ENGINE_2
         c.module_number = PHP_USER_CONSTANT;
-#endif
         zend_register_constant(&c TSRMLS_CC);
 
         zend_hash_move_forward_ex(Z_ARRVAL_P(constants), &pos);
@@ -821,15 +1002,7 @@ PHP_FUNCTION(apc_load_constants) {
 
     if(!strkey_len) RETURN_FALSE;
 
-#if PHP_API_VERSION < 20041225
-#if HAVE_APACHE && defined(APC_PHP4_STAT)
-    t = ((request_rec *)SG(server_context))->request_time;
-#else 
-    t = time(0);
-#endif
-#else 
     t = sapi_get_request_time(TSRMLS_C);
-#endif
 
     entry = apc_cache_user_find(apc_user_cache, strkey, strkey_len + 1, t);
 
@@ -850,10 +1023,9 @@ PHP_FUNCTION(apc_compile_file) {
     int filename_len;
     zend_file_handle file_handle;
     zend_op_array *op_array;
-    long slam_defense = 0;
     char** filters = NULL;
     zend_bool cache_by_default = 1;
-    HashTable cg_function_table, cg_class_table, eg_function_table, eg_class_table;
+    HashTable cg_function_table, cg_class_table;
     HashTable *cg_orig_function_table, *cg_orig_class_table, *eg_orig_function_table, *eg_orig_class_table;
 
     if(!APCG(enabled)) RETURN_FALSE;
@@ -864,23 +1036,13 @@ PHP_FUNCTION(apc_compile_file) {
 
     if(!filename) RETURN_FALSE;
 
-    /* If slam defense is active, temporarily disable */
-    if(APCG(slam_defense)) {
-        slam_defense = APCG(slam_defense);
-        APCG(slam_defense) = 0;
-    }
-   
-    /* If filter is active, temporarily disable */
-    if(APCG(filters) != NULL) {
-        filters = APCG(filters);
-        APCG(filters) = NULL;
-    }
+    /* reset filters and cache_by_default */
 
-    /* If cache_by_default is off, temporarily enable */
-    if(!APCG(cache_by_default)) {
-        cache_by_default = APCG(cache_by_default);
-        APCG(cache_by_default) = 1;
-    }
+    filters = APCG(filters);
+    APCG(filters) = NULL;
+
+    cache_by_default = APCG(cache_by_default);
+    APCG(cache_by_default) = 1;
 
     /* Replace function/class tables to avoid namespace conflicts */
     zend_hash_init_ex(&cg_function_table, 100, NULL, ZEND_FUNCTION_DTOR, 1, 0);
@@ -894,7 +1056,7 @@ PHP_FUNCTION(apc_compile_file) {
     eg_orig_class_table = EG(class_table);
     EG(class_table) = CG(class_table);
     APCG(force_file_update) = 1;
-    
+
     /* Compile the file, loading it into the cache */
     file_handle.type = ZEND_HANDLE_FILENAME;
     file_handle.filename = filename;
@@ -915,9 +1077,8 @@ PHP_FUNCTION(apc_compile_file) {
     zend_hash_destroy(&cg_class_table);
     EG(function_table) = eg_orig_function_table;
     EG(class_table) = eg_orig_class_table;
-    
+
     /* Restore global settings */
-    APCG(slam_defense) = slam_defense;
     APCG(filters) = filters;
     APCG(cache_by_default) = cache_by_default;
 
@@ -925,46 +1086,59 @@ PHP_FUNCTION(apc_compile_file) {
 
     /* Free up everything */
     zend_destroy_file_handle(&file_handle TSRMLS_CC);
-#ifdef ZEND_ENGINE_2
     destroy_op_array(op_array TSRMLS_CC);
-#else
-    destroy_op_array(op_array);
-#endif
     efree(op_array);
 
     RETURN_TRUE;
 }
 /* }}} */
 
+/* {{{ arginfo */
+ZEND_BEGIN_ARG_INFO_EX(php_apc_fetch_arginfo, 0, 0, 1)
+    ZEND_ARG_INFO(0, key)
+    ZEND_ARG_INFO(1, success)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(php_apc_inc_arginfo, 0, 0, 1)
+    ZEND_ARG_INFO(0, key)
+    ZEND_ARG_INFO(0, step)
+    ZEND_ARG_INFO(1, success)
+ZEND_END_ARG_INFO()
+/* }}} */
+
 /* {{{ apc_functions[] */
 function_entry apc_functions[] = {
-	PHP_FE(apc_cache_info,          NULL)
-	PHP_FE(apc_clear_cache,         NULL)
-	PHP_FE(apc_sma_info,            NULL)
-	PHP_FE(apc_store,               NULL)
-	PHP_FE(apc_fetch,               NULL)
-	PHP_FE(apc_delete,              NULL)
-	PHP_FE(apc_define_constants,    NULL)
-	PHP_FE(apc_load_constants,      NULL)
-	PHP_FE(apc_compile_file,        NULL)
-	PHP_FE(apc_add,                 NULL)
-	{NULL, 		NULL,				NULL}
+    PHP_FE(apc_cache_info,          NULL)
+    PHP_FE(apc_clear_cache,         NULL)
+    PHP_FE(apc_sma_info,            NULL)
+    PHP_FE(apc_store,               NULL)
+    PHP_FE(apc_fetch,               php_apc_fetch_arginfo)
+    PHP_FE(apc_delete,              NULL)
+    PHP_FE(apc_delete_file,         NULL)
+    PHP_FE(apc_define_constants,    NULL)
+    PHP_FE(apc_load_constants,      NULL)
+    PHP_FE(apc_compile_file,        NULL)
+    PHP_FE(apc_add,                 NULL)
+    PHP_FE(apc_inc,                 php_apc_inc_arginfo)
+    PHP_FE(apc_dec,                 php_apc_inc_arginfo)
+    PHP_FE(apc_cas,                 NULL)
+    {NULL,    NULL,                 NULL}
 };
 /* }}} */
 
 /* {{{ module definition structure */
 
 zend_module_entry apc_module_entry = {
-	STANDARD_MODULE_HEADER,
-	"apc",
-	apc_functions,
-	PHP_MINIT(apc),
-	PHP_MSHUTDOWN(apc),
-	PHP_RINIT(apc),
-	PHP_RSHUTDOWN(apc),
-	PHP_MINFO(apc),
-	APC_VERSION,
-	STANDARD_MODULE_PROPERTIES
+    STANDARD_MODULE_HEADER,
+    "apc",
+    apc_functions,
+    PHP_MINIT(apc),
+    PHP_MSHUTDOWN(apc),
+    PHP_RINIT(apc),
+    PHP_RSHUTDOWN(apc),
+    PHP_MINFO(apc),
+    PHP_APC_VERSION,
+    STANDARD_MODULE_PROPERTIES
 };
 
 #ifdef COMPILE_DL_APC
@@ -977,6 +1151,6 @@ ZEND_GET_MODULE(apc)
  * tab-width: 4
  * c-basic-offset: 4
  * End:
- * vim600: expandtab sw=4 ts=4 sts=4 fdm=marker
+ * vim>600: expandtab sw=4 ts=4 sts=4 fdm=marker
  * vim<600: expandtab sw=4 ts=4 sts=4
  */
